use Module::Build;

$build = Module::Build->new(
	create_makefile_pl => 'passthrough',
	dist_abstract      => 'SOAP with WSDL support',
	dist_name          => 'SOAP-WSDL',
	dist_version       => '2.00_16',
	module_name        => 'SOAP::WSDL',
	license            => 'artistic',
	requires           => {
		'Class::Std'              => q/v0.0.8/,
		'Class::Std::Storable'    => 0,
		'Date::Parse'             => 0,
		'Date::Format'            => 0,
		'LWP::UserAgent'          => 0,
		'List::Util'              => 0,
		'File::Basename'          => 0,
		'File::Path'              => 0,
		'XML::Parser::Expat'      => 0,
		'Template'                => 0,
		'Term::ReadKey'           => 0,
		'Getopt::Long'            => 0,
	},
	buildrequires => {
		'Date::Parse'             => 0,
		'Date::Format'            => 0,
		'Test::More'              => 0,
		'Class::Std'              => q/v0.0.8/,
		'Class::Std::Storable'    => 0,
		'List::Util'              => 0,
		'LWP::UserAgent'          => 0,
		'File::Basename'          => 0,
		'File::Path'              => 0,
		'XML::Parser::Expat'      => 0,
		'Template'                => 0,	
		'Getopt::Long'            => 0,
		'Cwd'                     => 0,
		'File::Find'              => 0,
		'Storable'                => 0,
	},
	recursive_test_files => 1,
);
$build->add_build_element('tt');
$build->create_build_script;
